
##############################################################################
#
# Copyright (c) 2005-2006 Balazs Ree <ree@greenfinity.hu> All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

'''\
Generate form submission
'''

import base64
from Products.mechagen.generator.forms import FileUpload
from base import SelectFormMixin, Action

class SubmitFileUpload:
    'File upload in the submit action'

    def __init__(self, control, index, value):
        self.name = control.name
        self.index = index
        assert isinstance(value, FileUpload)
        self.value = value

    def generate(self, writer, action):
        control = action.generate_find_control(writer, self.name, self.index)
        writer.write('# The generated upload_file can be deleted and replaced by an arbitrary file-like object.')
        writer.newline()
        encoded_lines = base64.encodestring(self.value.contents).splitlines(True)
        if not encoded_lines:
            # care for empty contents
            encoded_lines = ['\n'] 
        writer.write('upload_file =  StringIO(base64.decodestring(%s' % repr(encoded_lines[0]))
        writer.indenton()
        for line in encoded_lines[1:]:
            writer.write(' \\')
            writer.newline()
            writer.write(repr(line))
        writer.write('))')
        writer.indentoff()
        writer.newline()
        writer.write('%s.add_file(upload_file, content_type="%s", filename="%s")' % \
            (control, self.value.content_type, self.value.filename))
        writer.newline()
 
class SubmitDiff:
    'Difference in the submit action'

    def __init__(self, control, index, value):
        self.name = control.name
        self.disabled = control.disabled
        self.readonly = control.readonly
        self.index = index
        self.value = value

    def generate(self, writer, action):
        control = action.generate_find_control(writer, self.name, self.index)
        # If the control is readonly or disabled, we must set that attribute
        # or setting the value will fail.
        if self.readonly:
            writer.write('%s.readonly = False' % (control, ))
            writer.newline()
        if self.disabled:
            writer.write('%s.disabled = False' % (control, ))
            writer.newline()
        # Now generate the actual setting
        writer.write('%s.value = %s' % (control, repr(self.value)))
        writer.newline()

class FormSubmitAction(SelectFormMixin, Action):
    
    def __init__(self, data, all_forms, nr):

        self.set_form_data(all_forms, nr)

        # --
        # Compare values between the form and the data
        # --
        
        diffs = self.diffs = []
        
        def diffelem(control, index, value):
            if isinstance(value, FileUpload):
                # Handle file uploads (XXX We do not handle multiple uploads, 
                # - are they actually handled at all by Zope?)
                # We skip uploads with an empty contents.
                if value.contents:
                    diffs.append(SubmitFileUpload(control, index, value))
            else:
                # Normal form controls. If it's a multiline control, we must
                # compare by splitlines, because we don't want the \r\n != \n
                # and the ending newline to make a difference.
                # XXX The ending newline could actually be a difference, but
                # it seems that the browsers swallow that on submit?
                # so we currently ignore it
                def cleanvalue(t):
                    if isinstance(t, basestring):
                        return '\n'.join(t.splitlines())
                    else:
                        # a composite control value
                        return t
                if cleanvalue(control.value) != cleanvalue(value):
                    ##print '%s : "%s" != "%s"' % (control.name, control.value, value)
                    diffs.append(SubmitDiff(control, index, value))

        # We iterate on all the elems in the new form, and see what's up
        # with the corresponding controls.
        for (key, rkey), value in data.form.encoded_arrays.iteritems():
            
            encoded_key = data.form.encoded_name(key, rkey)
            
            # look up the controls in the form
            controls = self.b_form.find_controls(name=encoded_key)

            if not controls:
                ##print 'Notfound', key
                # but we ignore this, this should not happen
                ## XXX However, now it can happen: since the generation of
                ## the FormSubmitAction can be "dropped"... in this case we
                ## can get this error if exactly this parm was specified
                ## on the url, so it was missing from the form.
                ## But then this is not a submit action.
                continue

            if not isinstance(value, list):
                # just one elem
                value = [value]

            # See if the control is a composite: in this case its
            # value will be a list (checkboxes)
            if isinstance(controls[0].value, list):
                diffelem(controls[0], 0, value)
                # I hope there are no more controls, I ignore them anyway.
            else:
                # we diff the elems one by one
                for i, control in enumerate(controls):
                    try:
                        v = value[i]
                    except IndexError:
                        # Bad luck...
                        break
                    diffelem(control, i, v)
                    
        # We now went by the new form data. So the only thing left is to go
        # through it again by control, and see if for some control there is no
        # data in the new form.
        # This happens, for example, if all the rows of a checkbox array are unchecked.
        for control in self.b_form.controls:
            # these can be others, too. But we only consider lists.
            if control.name not in dict(data.form.encoded_form) and \
                    isinstance(control.value, list):
                diffelem(control, 0, [])
                    
        # --
        # now collect the submit data
        # --
        submit_controls = self.b_form.find_controls(predicate = lambda control: 
            control.type in ('submit', 'button'))
        encoded_form_dict = dict(data.form.encoded_form)
        for control in submit_controls:
            # only the real submit button will be in the form.
            if control.name in encoded_form_dict:
                # ok, this was the submit control
                self.submit_control_name = control.name
                break
        else:
            # this is an error already... no control to submit
            # XXX the whole action will be dropped from outside!!!
            self.submit_control_name = None
            
    def generate_set(self, writer):
        if self.diffs:
            self.gen_select_form(writer, do_remark=True)
        for diff in self.diffs:
            diff.generate(writer, self)
        self.gen_select_form(writer, do_remark=True)

    def generate_submit(self, writer):
        writer.write('page = %s.submit("%s")' % (writer.browservar, self.submit_control_name))
        writer.newline()
