
##############################################################################
#
# Copyright (c) 2005-2006 Balazs Ree <ree@greenfinity.hu> All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

'''\
Forms internal representation
'''

from utils import lazy

class FileUpload(object):
    'Class to wrap contents of a ZPublisher FileUpload'
    
    def __init__(self, value):
        # take out what's nesessary from the request
        self.headers = value.headers
        self.content_type = value.headers.get('Content-Type', None)
        self.filename = value.filename
        # this is why we are doing this wrapping: we need to save
        # the file contents or else it would be "gone".
        value.seek(0)
        self.contents = value.read()
        value.seek(0)
    
class Forms(dict):

    def __init__(self, d={}):
        # FileUpload values must be stubbed right away, to keep submit data
        from ZPublisher import HTTPRequest
        _d = dict(d)
        for key, value in _d.iteritems():
            if isinstance(value, HTTPRequest.FileUpload):
                _d[key] = FileUpload(value)
        dict.__init__(self, _d)
        
    # XXX XXX What's up with HTTPRequest.Record types?
    
    def _encode_item(key, value):
        forms, arrays, modifiers = [], {}, []
        if isinstance(value, (list, tuple)) and value:
            for row in value:
                try:
                    items = row.items()
                except:
                    # :list
                    # error... what to do... add as a list. 
                    # XXX these could be mixed in case of an error, ... quite bad?
                    forms.append(('%s:list' % (key, ), row)) 
                    try:
                        array = arrays[key, None]
                    except KeyError:
                        array = arrays[key, None] = []
                    array.append(row)
                    master_modifier = 'list'
                else:
                    # :records
                    for rkey, rvalue in items:
                        forms.append(('%s.%s:records' % (key, rkey), rvalue))
                        try:
                            array = arrays[key, rkey]
                        except KeyError:
                            array = arrays[key, rkey] = []
                        array.append(rvalue)
                    master_modifier = 'records'
        else:
            try:
                items = value.items()
            except:
                # fallback, possibly can be converted into string.
                # try, might be good or might not
                forms.append((key, value))
                arrays[key, None] = value
                master_modifier = None
            else:
                # record
                for rkey, rvalue in items:
                    forms.append(('%s.%s:record' % (key, rkey), rvalue))
                    arrays[key, rkey] = rvalue
                master_modifier = 'record'
        if master_modifier:
            modifiers.insert(0, master_modifier)
        return forms, arrays, modifiers
    _encode_item = staticmethod(_encode_item)
    
    def _generate_encoded(self):
        '''Lazy generate function
        
        Encoded form values
        -------------------
        
        We get the form values ":"-encoded. We have to 
        convert them back.

        e.g.
            'xxx': 'a' will get [('xxx', 'a')]
            'xxx': ['a', 'b', 'c'] will get [('xxx:list', 'a'), ('xxx:list', 'b'), ('xxx:list', 'c')]
            'xxx': {'k': 'a', 'l': 'b'} will get [('xxx.k:record', 'a'), ('xxx.l:record', 'b')]
            'xxx': [{'k': 'a', 'l': 'b'}, {'k': 'c'}] will get [('xxx.k:records', 'a'), 
                                                ('xxx.l:records', 'b'), ('xxx.k:records', 'c')]

        Encoded arrays
        --------------

        The encoded arrays are dictionaries with the (key, rkey) as keys,
        and single elements or arrays as the values.

        e.g.
            'xxx': 'a' will get {('xxx', None): 'a'}
            'xxx': ['a', 'b', 'c'] will get {('xxx', None): ['a', 'b', 'c']}
            'xxx': {'k': 'a', 'l': 'b'} will get {('xxx', 'k'): 'a','xxx.l': 'b' }
            'xxx': [{'k': 'a', 'l': 'b'}, {'k': 'c'}] will get {('xxx', 'k'): ['a', 'c'], 'xxx.l': ['b'] }
        
        
        Modifiers
        ---------
        
        A list of encoding modifiers (by key). Currently only list, record, and records
        and the result will be a one-element (or empty) list.

        e.g.
            'xxx': 'a' will get {'xxx': []}
            'xxx': ['a', 'b', 'c'] will get {'xxx': ['list']}
            'xxx': {'k': 'a', 'l': 'b'} will get {'xxx': ['record']}
            'xxx': [{'k': 'a', 'l': 'b'}, {'k': 'c'}] will get {'xxx': ['records']} 
        '''
        encoded_form = []
        encoded_arrays = {}
        encoded_modifiers = {}
        for key, value in self.iteritems():
            form, arrays, modifiers = self._encode_item(key, value)
            encoded_form.extend(form)
            encoded_arrays.update(arrays)
            encoded_modifiers[key] = modifiers
        return encoded_form, encoded_arrays, encoded_modifiers
     
    encoded_form, encoded_arrays, encoded_modifiers = lazy(_generate_encoded, 3)

    def encoded_name(self, key, rkey=None):
        'Gets the full name for a key and an optional record key'
        modifiers = self.encoded_modifiers[key]
        try:
            master_modifier = modifiers[0]
        except IndexError:
            postfix = ''
        else:
            postfix = ':' + master_modifier
        if rkey is not None:
            key = '%s.%s' % (key, rkey)
            assert master_modifier in ('record', 'records')
        return key + postfix
