
import unittest
from Products.mechagen.generator.forms import Forms

class TestForms(unittest.TestCase):
    'Test form class'
    
    def test_basic(self):
        'works with different (single) items'

        f = Forms({'xxx': 'a'})
        self.assertEqual(f.encoded_form, [('xxx', 'a')])
        self.assertEqual(f.encoded_arrays, {('xxx', None): 'a'})
        self.assertEqual(f.encoded_modifiers, {'xxx': []})
        self.assertEqual(f.encoded_name('xxx'), 'xxx')

        f = Forms({'xxx': ['a', 'b', 'c']})
        self.assertEqual(f.encoded_form, [('xxx:list', 'a'), ('xxx:list', 'b'), ('xxx:list', 'c')])
        self.assertEqual(f.encoded_arrays, {('xxx', None): ['a', 'b', 'c']})
        self.assertEqual(f.encoded_modifiers, {'xxx': ['list']})
        self.assertEqual(f.encoded_name('xxx'), 'xxx:list')
 
        f = Forms({'xxx': {'k': 'a', 'l': 'b'}})
        self.assertEqual(f.encoded_form, [('xxx.k:record', 'a'), ('xxx.l:record', 'b')])
        self.assertEqual(f.encoded_arrays, {('xxx', 'k'): 'a',('xxx', 'l'): 'b' })
        self.assertEqual(f.encoded_modifiers, {'xxx': ['record']})
        self.assertEqual(f.encoded_name('xxx', 'k'), 'xxx.k:record')

        f = Forms({'xxx': [{'k': 'a', 'l': 'b'}, {'k': 'c'}]})
        self.assertEqual(f.encoded_form, [('xxx.k:records', 'a'),  ('xxx.l:records', 'b'), ('xxx.k:records', 'c')])
        self.assertEqual(f.encoded_arrays, {('xxx', 'k'): ['a', 'c'], ('xxx', 'l'): ['b'] })
        self.assertEqual(f.encoded_modifiers, {'xxx': ['records']})
        self.assertEqual(f.encoded_name('xxx', 'k'), 'xxx.k:records')

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestForms))
    return suite
