
import unittest
from Products.mechagen.generator.writer import Writer
from cStringIO import StringIO

class TestWriter(unittest.TestCase):
    'Test writer'

    def setUp(self):
        self.w = Writer(StringIO())

    def equals(self, value):
        return self.assertEqual(self.w.stream.getvalue(), value)

    def test_newline(self):
        self.w.write('AAA')
        self.w.newline()
        self.w.indenton()
        self.w.write('BBB')
        self.w.newline()
        self.w.indentoff()
        self.w.write('CCC')
        self.w.newline()
        self.equals(
            'AAA\n' \
            '    BBB\n' \
            'CCC'
            )
        self.w.indentoff()
        self.w.write('DDD')
        self.equals(
            'AAA\n' \
            '    BBB\n' \
            'CCC\n' \
            'DDD'
            )

    def test_strips(self):
        # No strips within the same line 
        self.w.write('AAA')
        self.w.write(' BBB')
        self.equals(
            'AAA BBB'
            )
        # strips in a new line
        self.w.newline()
        self.w.write('  CCC')
        self.w.newline()
        self.equals(
            'AAA BBB\n' \
            'CCC'
            )
        # even more newlines
        self.w.newline()
        self.w.newline()
        self.equals(
            'AAA BBB\n' \
            'CCC'
            )
        self.w.write('DDD')
        self.equals(
            'AAA BBB\n' \
            'CCC\n' \
            '\n' \
            '\n' \
            'DDD'
            )

    def test_multiline(self):
        self.w.indenton()
        self.w.write('AAA\nBBB')
        self.equals(
            '    AAA\n' \
            '    BBB'
            )
        # newline was not marked
        self.w.write('CCC')
        self.equals(
            '    AAA\n' \
            '    BBBCCC'
            )
        # this time newline is marked
        self.w.newline()
        self.w.write('DDD\nEEE\n')
        self.w.write('FFF')
        self.equals(
            '    AAA\n' \
            '    BBBCCC\n' \
            '    DDD\n' \
            '    EEE\n' \
            '    FFF'
            )

    def test_dedents(self):
        # No dedents within the same line 
        self.w.write('AAA')
        self.w.write('  BBB\n  CCC')
        self.equals(
            'AAA  BBB\n' \
            '  CCC'
            )
        # dedents multiple line
        self.w.newline()
        self.w.write('  DDD\n  EEE')
        self.equals(
            'AAA  BBB\n' \
            '  CCC\n' \
            'DDD\n' \
            'EEE'
            )
        # dedents multiple line (really a dedent)
        self.w.newline()
        self.w.write('      FFF\n  GGG')
        self.w.newline()
        self.equals(
            'AAA  BBB\n' \
            '  CCC\n' \
            'DDD\n' \
            'EEE\n' \
            '    FFF\n' \
            'GGG'
            )

    def test_multiline_indents(self):
        # Multiline indents are preserved after dedent 
        self.w.write('AAA\n  BBB\n  CCC')
        self.equals(
            'AAA\n' \
            '  BBB\n' \
            '  CCC'
            )

    def test_remark(self):
        self.w.indenton()
        self.w.indenton()
        self.assertEqual(self.w.add_remark('AAAA', 'BBBB'), 'AAAA' + ' '*(65-8-4) + '# BBBB')
        self.assertEqual(self.w.add_remark('A'*70, 'BBBB'), 'A'*70 + '  # BBBB')
        self.assertEqual(self.w.add_remark('A'*58, 'BBBB'), 'A'*58 + '  # BBBB')
        self.assertEqual(self.w.add_remark('A'*57, 'BBBB'), 'A'*57 + '  # BBBB')
        self.assertEqual(self.w.add_remark('A'*56, 'BBBB'), 'A'*56 + '  # BBBB')
        self.assertEqual(self.w.add_remark('A'*55, 'BBBB'), 'A'*55 + '  # BBBB')
        self.assertEqual(self.w.add_remark('A'*54, 'BBBB'), 'A'*54 + '   # BBBB')
        self.assertEqual(self.w.add_remark('A'*53, 'BBBB'), 'A'*53 + '    # BBBB')
        self.assertEqual(self.w.add_remark('A'*52, 'BBBB'), 'A'*52 + '     # BBBB')
        
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestWriter))
    return suite
