
##############################################################################
#
# Copyright (c) 2005-2006 Balazs Ree <ree@greenfinity.hu> All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

'''\
Writer / Formatting
'''

import re
from textwrap import dedent

class Writer:

    def __init__(self, stream, baseindent=4):
        self.stream = stream
        self.indent = 0
        self.baseindent = baseindent
        self.isnewline = True
        self.nrnewlines = 0

    re_multi = re.compile(r'(\n)')
    
    def write(self, txt, keepindents=False):
        if self.isnewline:
            self.stream.write('\n' * self.nrnewlines)
            self.stream.write(' ' * self.indent)
            self.isnewline = False 
            self.nrnewlines = 0
        else:
            # if a continuation line, then we always keep indents.
            keepindents = True
        if not keepindents:
            # (new lines are getting dedented, continuations not.)
            txt = dedent(txt)
        # see if there are more lines
        lines = self.re_multi.split(txt)
        if len(lines) > 1:
            # more lines
            for line in lines:
                if line == '\n':
                    self.newline()
                else:
                    self.write(line, keepindents=True)
        else:
            # single line
            self.stream.write(txt)

    def newline(self):
        self.isnewline = True
        self.nrnewlines += 1

    def indenton(self, levels=1):
        self.indent += self.baseindent * levels
        
    def indentoff(self, levels=1):
        self.indent -= self.baseindent * levels

    def add_remark(self, text, remark, pos=65):
        if remark is not None:
            # remark will be indented to position, or later if text is longer.
            return text.ljust(pos - 2  - self.indent) + '  # ' + remark
        else:
            return text
